# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.protocol.base import BaseProtocol
from driverng.utils import ACK, EOT, ETX, NACK, STX, byte2log


class TosohProtocol(BaseProtocol):
    def __init__(self, max_tries=4, **kwargs):
        self.max_tries = max_tries
        super(TosohProtocol, self).__init__(**kwargs)

    def send_trames(self, trames):
        for line in trames:
            self.tries = 0
            while True:
                self.tries += 1
                self.write(line)
                data = self.read(timeout=10)
                if not data or data[0] not in (ACK, NACK):
                    self.logger.info("Unexpected data '%s'", byte2log(data))
                    self.write(EOT)
                    return False
                if data[0] == ACK:
                    break
                if self.tries >= self.max_tries:
                    self.write(EOT)
                    return False
        self.write(EOT)
        return True

    def recv_trames(self):

        lines = []

        while True:
            data = self.read()

            if not data or data[0] not in (STX, EOT):
                self.logger.warning("<stx> or <eot> expexted in %s", byte2log(data))
                return False

            if data[0] == EOT:
                break

            data += self.readline(
                break_func=lambda s: s and len(s) >= 2 and s[-2] == ETX
            )
            if self.check_trame(data):
                self.write(ACK)
                lines.append(self.get_payload(data))
            else:
                self.write(NACK)
                return False

        messages = []
        current = []
        for line in lines:
            if line and bytearray([line[0]]) == b"1" and current:
                messages.append(b"\n".join(current))
                current = []

            current.append(line)

        if current:
            messages.append(b"\n".join(current))

        return messages

    def checksum(self, data):
        # XOR of <payload><etx>
        ck = 3  # ETX
        for c in bytearray(data):
            ck ^= c
        return ck

    def check_trame(self, trame):
        return True

    def get_payload(self, data):
        # <stx><payload><etx><checksum>
        return data[1:-2]

    def build_trames(self, message):
        # <stx><payload><etx><checksum>
        data = []
        for line in message.splitlines():
            data.append(b"%c%s%c%c" % (STX, line, ETX, self.checksum(line)))
        return data
